/*------------------------------------------------------------------------------------
   Name: Matrix-Signals.mq4
   Author: Xaphod. 2012, xaphod.com
   
   Description: Signal/alert generating indicator for use with the 'Genesis Matrix
                Trading System'
   Requires:  experts/indicators/TVI_mq4
              experts/indicators/GannHiLo-Histo.mq4
              experts/indicators/T3MA-Basic.mq4
              experts/libraries/MatrixBridge.dll
              experts/files/MatrixSignals.exe
              experts/files/MatrixAlert.wav 
   Change log:
       2014-02-24. Xaphod, v1.601
          - Use TERMINL_DATA_PATH to locate files instead of TERMINAL_PATH
            Affects Matrix-Signals.mq4 and Matrix-Signals.dll
       2014-02-14. Xaphod, v1.600
          - Update to mql4.5 / MT4.5
          - Removed check for custom indicators. There is no way to find the exact path 
            of the indicators as they can be in sub-folders in the indicators folder.
       2012-11-11. Xaphod, v1.01
          - Added Symbol Prefix parameter.
          - Remove symbol prefix, suffix and non alphanumeric chars from symbol string 
            sent to alert program.
          - Adjust the size of the alert window to account for the desktop font size 
            variations.
          - Read out 5 digit symbols.
          - Check if all files are installed on start up and check for dlls enabled.
          - Improved symbol validation function. 
          - Added blink time parameter in '../experts/files/MatrixSignals.ini' to 
            adjust the alert blink time. default BlinkTime=60. One will have to edit
            the ini file to change the value.

       2012-08-31. Xaphod, v1.00
          - Initial release
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright 2012, Xaphod"
#property link      "http://www.xaphod.com"
#property strict
#property version    "1.601"
#property description "Genesis Matrix Signal/Alert Indicator for use with the 'Genesis Matrix Trading System'"      
#property description " "
#property description "Requirements: The following indicators must installed and working:"
#property description "  /Indicators/TVI.mq4, /Indicators/GannHiLo-Histo.mq4, /Indicators/T3MA-Basic.mq4,"
#property description "  /Libraries/MatrixBridge.dll, /Files/MatrixSignals.exe, /Files/MatrixAlert.wav "
#property description " "
#property description "Note: Use only one instance of the program per MT4 installation. Ie. run only on one chart."
#property description " "
#property description "Troubleshooting: Check the 'Journal' and 'Expert' tabs in the terminal window for errors."
#property indicator_chart_window

//#include <xPrint.mqh>

// Win32 API
#import "Kernel32.dll"
  int GetFileAttributesW (string lpFileName);
#import

#import "MatrixBridge.dll"
  int gmaInit(int hChartWnd, string sDataPath);
  int gmaDeinit(int hChartWnd);
  int gmaSymbolAdd(string szSymbol);  // changed
  int gmaSymbolUpdate(int iSymbolId, int iSignal);
#import

// Constant definitions
#define INDICATOR_NAME "Matrix-Signals"

#define IND_TVI           "TVI"
#define IND_GANNHILO      "GannHiLo-Histo"
#define IND_T3MA          "T3MA-Basic"
#define FILE_ALERTBRIDGE  "MatrixBridge.dll"
#define FILE_ALERTPROGRAM "MatrixSignals.exe"
#define FILE_ALERTSND     "MatrixAlert.wav"

// Signals
#define SIG_UP 1
#define SIG_NO 0
#define SIG_DN -1

#define MAX_SYMBOLS 20

// Indicator parameters
extern ENUM_TIMEFRAMES TimeFrame_Period=PERIOD_CURRENT; /* TimeFrame Period */              // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0. 
extern string    Symbol_List="EURUSD, EURJPY, AUDUSD,GBPUSD";  /* Symbol List (comma separated) */ // List of symbols to monitor. , without the brokers prefix and suffix.
extern string    Symbol_Suffix="";   /* Symbol Suffix (Ex: m for EURUSDm) */   // Broker symbol suffix. Ex m for EURUSDm.
extern string    Symbol_Prefix="";   /* Symbol Prefix (Ex: fx_ for fx_EURUSD) */    // Broker symbol prefix. Ex fx_ for fx_EURUSD.
extern int       TVI_r=12; /* TVI r */
extern int       TVI_s=12; /* TVI s */
extern int       TVI_u=5;  /* TVI u */
extern int       CCI_Period=20;  /* CCI Period */
extern int       GannHiLo_Period=10;  /* GannHiLo Period */
extern int       T3MA_Period=8;      /* T3MA Period */
extern double    T3MA_b=0.618;       /* T3MA b */

// Globals
int ghChartWnd;


//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {  
  if (!IsDllsAllowed())
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+", \'Allow Dll imports\' is required for this indicator to work. ");

  // Check if external indicators and files are installed
  if (!FileExists(TerminalInfoString(TERMINAL_DATA_PATH)+"\\MQL4\\Libraries\\"+FILE_ALERTBRIDGE))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalInfoString(TERMINAL_DATA_PATH)+"\\MQL4\\Libraries\\"+FILE_ALERTBRIDGE+" cannot be found.");
  if (!FileExists(TerminalInfoString(TERMINAL_DATA_PATH)+"\\MQL4\\Files\\"+FILE_ALERTPROGRAM))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalInfoString(TERMINAL_DATA_PATH)+"\\MQL4\\Files\\"+FILE_ALERTPROGRAM+" cannot be found.");
  if (!FileExists(TerminalInfoString(TERMINAL_DATA_PATH)+"\\MQL4\\Files\\"+FILE_ALERTSND))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalInfoString(TERMINAL_DATA_PATH)+"\\MQL4\\Files\\"+FILE_ALERTSND+" cannot be found.");
 
  // Sanitize string inputs
  Symbol_List=StringTrimRight(StringTrimLeft(Symbol_List));
  Symbol_Suffix=StringTrimRight(StringTrimLeft(Symbol_Suffix));
  Symbol_Prefix=StringTrimRight(StringTrimLeft(Symbol_Prefix)); 
  
  // Misc
  IndicatorShortName(INDICATOR_NAME);
  ghChartWnd=0;
  
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
  // unload dll
  gmaDeinit(ghChartWnd);
  return (0);
}


//-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  static string saSymbol[MAX_SYMBOLS];
  static int iaSymbol[MAX_SYMBOLS];
  static int iaSignal[MAX_SYMBOLS];
  static int iaBars[MAX_SYMBOLS];
  static int iSymbolCount=0;
  int i,iNewBars;


  // Init symbols and load GUI;
  if (iSymbolCount==0) {
    // Check if chart is running
    if (IsStopped())
      return(0);
    ghChartWnd=WindowHandle(Symbol(), Period());  
    if (ghChartWnd==0)
      return(0);
     
    // Load symbols
    iSymbolCount=LoadSymbols(Symbol_List, Symbol_Prefix, Symbol_Suffix, saSymbol, iaSymbol); 
    // Error. No symbols found
    if (iSymbolCount==0) {
      Alert(INDICATOR_NAME+", "+Symbol()+". Error! No Symbols Found");
      iSymbolCount--;
      return(0);
    }
    // Start GUI and add symbols to it  
    else {
      if (gmaInit(ghChartWnd,TerminalInfoString(TERMINAL_DATA_PATH))>0) {
        for (i=0; i<iSymbolCount; i++) {
                  iaSymbol[i]= gmaSymbolAdd(SanitizeSymbol(saSymbol[i])); // changed
        }
      }
      else {
        Alert(INDICATOR_NAME+", "+Symbol()+". Error! Unable to start GUI");
        iSymbolCount--;
        return(0);
      }
    }
  }
  
  // Get Signals
  if (iSymbolCount>0) {
    for (i=0;i<iSymbolCount;i++) {
      iNewBars=NewBars(TimeFrame_Period, i, iaBars, saSymbol);
      if (iNewBars>0) {
        iaSignal[i]=GetCurrentSignal(saSymbol[i],TimeFrame_Period);
        gmaSymbolUpdate(iaSymbol[i],iaSignal[i]);
      }      
    }
  }
  
  return(0);
}

//+------------------------------------------------------------------+


//-----------------------------------------------------------------------------
// function: LoadSymbols()
// Description: Load the symbols from list provided
//-----------------------------------------------------------------------------
int LoadSymbols(string sSymbolList, string sSymbolPrefix, string sSymbolSuffix, string& saSymbol[], int& iaSymbol[] ) {
  int iSymbolCount=0,i=0;
  string sCurSymbol;
  
  
  //Get list of symbols to use
  while (StringLen(sSymbolList)>0 && i<MAX_SYMBOLS) {
    sCurSymbol=StringTrimLeft(StringTrimRight(ParseStr(sSymbolList)));
    // Add symbol from parameters to symbol list
    if (StringLen(sCurSymbol)>0) {
      if (IsValidSymbol(sSymbolPrefix+sCurSymbol+sSymbolSuffix)) {
        saSymbol[i]=sSymbolPrefix+sCurSymbol+sSymbolSuffix;
        i++;
        iSymbolCount++;
      }
      else {
        Alert(INDICATOR_NAME+", "+Symbol()+". Error! "+sCurSymbol+sSymbolSuffix+" is not a valid Symbol_");
      }  
    }  
  }
  return(iSymbolCount);  
}


//-----------------------------------------------------------------------------
// function: GetCurrentSignals()
// Description: Load the current signals into the signal array
//-----------------------------------------------------------------------------
int GetCurrentSignal(string sSymbol, int iTimeFrame){
  int i;
  int iCurTVI, iCurCCI, iCurGHL, iCurT3;
  double dUp, dSig1, dSig2;
  
  // TVI
  dSig1=iCustom(sSymbol,iTimeFrame,IND_TVI,TVI_r,TVI_s,TVI_u,0,1);
  dSig2=iCustom(sSymbol,iTimeFrame,IND_TVI,TVI_r,TVI_s,TVI_u,0,2);
      
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurTVI=SIG_NO;
  else  if (dSig1>=dSig2)   
    iCurTVI=SIG_UP;
  else 
    iCurTVI=SIG_DN;
  
  // T3 MA
  dSig1 = iCustom(sSymbol,iTimeFrame,IND_T3MA,T3MA_Period, T3MA_b,300,0,1);
  dSig2 = iCustom(sSymbol,iTimeFrame,IND_T3MA,T3MA_Period, T3MA_b,300,0,2);
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurT3=SIG_NO;
  else if (dSig1>=dSig2) 
    iCurT3=SIG_UP;
  else
    iCurT3=SIG_DN;
  
  // Gann HiLo
  dUp = iCustom(sSymbol,iTimeFrame,IND_GANNHILO,GannHiLo_Period,0,1);
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurGHL=SIG_NO;
  else if (dUp>0) 
    iCurGHL=SIG_UP;
  else
    iCurGHL=SIG_DN;
  
  //Trend
  dSig1=iCCI(sSymbol, iTimeFrame, CCI_Period, PRICE_TYPICAL,1);
  if (iClose(sSymbol,iTimeFrame,1)==0 || iClose(sSymbol,iTimeFrame,1)==EMPTY_VALUE)
    iCurCCI=SIG_NO;
  else if (dSig1<0) 
    iCurCCI=SIG_DN;
  else
    iCurCCI=SIG_UP;
  
  // Set signal
  i=UpSignal(iCurTVI, iCurCCI, iCurGHL, iCurT3);
  if (i>2)
    return(i);
  i=DnSignal(iCurTVI, iCurCCI, iCurGHL, iCurT3);
  if (i<-2)
    return(i);  

  return(0);
}


//-----------------------------------------------------------------------------
// function: UpSignal()
// Description: 
//-----------------------------------------------------------------------------
int UpSignal(int iCurTVI, int iCurCCI, int iCurGHL, int iCurT3) {
  int n=0;
  if (iCurTVI==SIG_UP) n++;
  if (iCurCCI==SIG_UP) n++;
  if (iCurGHL==SIG_UP) n++;
  if (iCurT3==SIG_UP) n++;  
  return(n);
}


//-----------------------------------------------------------------------------
// function: DnSignal()
// Description: 
//-----------------------------------------------------------------------------
int DnSignal(int iCurTVI, int iCurCCI, int iCurGHL, int iCurT3) {
  int n=0;
  if (iCurTVI==SIG_DN) n--;
  if (iCurCCI==SIG_DN) n--;
  if (iCurGHL==SIG_DN) n--;
  if (iCurT3==SIG_DN) n--;  
  return(n);
}


//-----------------------------------------------------------------------------
// function: ParseStr()
// Description: Get next value in a comma delimited string
//-----------------------------------------------------------------------------
string ParseStr(string& sData, string sDelimiter=",") {
  int iPos;
  string sResult;
  
  iPos=StringFind(sData,sDelimiter,0);
  if (iPos==-1 && sData!="") {
    sResult=sData;
    sData="";
    return(sResult);
  }
  sResult=StringSubstr(sData,0,iPos);
  sData=StringSubstr(sData,iPos+1);
  return(sResult);
}


//-----------------------------------------------------------------------------
// function: NewBars()
// Description: Return nr of new bars on a TF
//-----------------------------------------------------------------------------
int NewBars(int iPeriod, int n, int& iaBars[], string& saSymbol[]) {
  int iNewSize;
  int iNewBars;
  datetime tTimeArray[];
  
  ArrayCopySeries(tTimeArray,MODE_TIME,saSymbol[n],iPeriod);
  iNewSize=ArraySize(tTimeArray);
  iNewBars=iNewSize-iaBars[n];
  iaBars[n]=iNewSize;
  return(iNewBars);
}


//-----------------------------------------------------------------------------
// function: FileExists()
// Description: Check if a file exists
//-----------------------------------------------------------------------------
bool FileExists(string sFileName) {
  // Exit if dlls are disabled
  if (!IsDllsAllowed())
    return(True);
  // Check file  
  Print(sFileName);
  if (GetFileAttributesW(sFileName)==-1) {
    return(False);    
  }
  return(True);
}


//-----------------------------------------------------------------------------
// function: IsValidSymbol()
// Description: Get symbol that contains the given symbol string by scanning
//              the file 'symbols.raw'. 
//              Returns true if symbol is found.
//-----------------------------------------------------------------------------
bool IsValidSymbol(string sSymbol) {
   int hSymbolFile;
   int iNrOfSymbols;
   string sResult;
   int i;
   
   hSymbolFile=FileOpenHistory("symbols.raw",FILE_BIN|FILE_READ);
   iNrOfSymbols=(int)FileSize(hSymbolFile)/1936;   
   for (i=0;i<iNrOfSymbols;i++) {
     if(FileIsEnding(hSymbolFile)) 
       break;
     sResult=StringTrimRight(FileReadString(hSymbolFile,12));
     if (sResult==sSymbol) {        
       FileClose(hSymbolFile);
       return(True); 
     }  
     sResult="";       
     FileSeek(hSymbolFile,1936-12,SEEK_CUR);     
   }
   FileClose(hSymbolFile);
   return(False);   
}


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+(string)iPeriod);
  }
  return("M?");
}


//-----------------------------------------------------------------------------
// function: StrToAlphaNum()
// Description: Convert a string to an alphanumeric string
//-----------------------------------------------------------------------------
string SanitizeSymbol(string sSymbol) {
  string sResult="";
  
  sResult=sSymbol;
  if (StringLen(Symbol_Suffix)>0)
    sResult=StringSubstr(sResult,0,StringLen(sResult)-StringLen(Symbol_Suffix));
  if (StringLen(Symbol_Prefix)>0)
   sResult=StringSubstr(sResult,StringLen(Symbol_Prefix),StringLen(sResult)-StringLen(Symbol_Prefix));
  sResult=StrToAlphaNum(sResult);
  
  return(sResult);
}


//-----------------------------------------------------------------------------
// function: StrToAlphaNum()
// Description: Convert a string to an alphanumeric string
//-----------------------------------------------------------------------------
string StrToAlphaNum(string sInStr) {
  string sResult="";
  int i;
  
  for (i=0; i<StringLen(sInStr);i++)
    if (IsAlphaNum(StringGetChar(sInStr,i)))
      sResult=sResult+StringSubstr(sInStr,i,1);
  
  return(sResult);
}


//-----------------------------------------------------------------------------
// function: IsDigit()
// Description: Is the character a digit?
//-----------------------------------------------------------------------------
bool IsDigit(int ch) {
  if (ch>='0' && ch<='9')
    return(true);
  else  
    return(False);
}


//-----------------------------------------------------------------------------
// function: IsAlpha()
// Description: Is the character an english letter? (A-Z and a-z)
//-----------------------------------------------------------------------------
bool IsAlpha(int ch) {
  if (ch>='A' && ch<='Z')
    return(true);
  else if (ch>='a' && ch<='z')
    return(true);
  else  
    return(False);
}    


//-----------------------------------------------------------------------------
// function: IsAlphaNum()
// Description: Is the character an alphanumeric character? (0-9, A-Z, a-z)
//-----------------------------------------------------------------------------
bool IsAlphaNum(int ch) {
  
  if (IsAlpha(ch) || IsDigit(ch))
    return(true);
  else 
    return(False);
} 